@@ -2,6 +2,7 @@
 # -*- coding: utf-8 -*-
 
 import sys, re, mechanize, base64, urllib2, time, calendar, urllib, sqlite3
+from lxml.html import soupparser
 from PyQt4 import QtCore, QtGui
 from PyQt4.QtGui import QApplication, QDialog, QImage, QPixmap
 from www2sms_gui_dialog_captcha import Ui_DialogCaptcha
@@ -18,6 +19,43 @@ class ShowCAPTCHA(QDialog, Ui_DialogCaptcha):
 		self.ui = Ui_DialogCaptcha()
 		self.ui.setupUi(self)
 
+def SearchCAPTCHA(self, baseurl, response, regex, element, attrib):
+	def gen_elements(tag, root):
+		if root.tag == tag:
+		    yield root
+		for child in root:
+		    for elt in gen_elements(tag, child):
+			yield elt
+
+	def gen_img_src(content):
+		content = soupparser.fromstring(content)
+		for elt in gen_elements(element, content):
+		    yield elt.attrib.get(attrib, None)
+
+	for src in gen_img_src(response):
+		#print src
+		if re.search(regex, src):
+			# Download the CAPTCHA and save it as a QPixmap
+			captcha_url = "%s%s" %(baseurl, src)
+			dl_captcha = urllib2.urlopen(captcha_url)
+			qimg = QImage.fromData(dl_captcha.read())
+			pixmap = QPixmap.fromImage(qimg)
+			break
+	sc = ShowCAPTCHA()
+
+	# Set captcha to show up in the CAPTCHA dialog
+	sc.ui.label_captcha.setPixmap(pixmap)
+	def getcaptcha():
+		self.ctxt = unicode(sc.ui.lineEdit_captcha.text())
+	QtCore.QObject.connect(sc.ui.buttonBox_Captcha, QtCore.SIGNAL("accepted()"), getcaptcha)
+	sc.show()
+	sc.exec_()
+	try:
+		return self.ctxt
+	except:
+		return "canceled"
+
+
 def error(self, MainWindow, exception, user_msg):
 	# Just for error handling stuff.
 	# When this module exits with "True" the user will see a notification with user_msg
@@ -151,6 +189,7 @@ class provider_o2online_de(provider):
 
 			#Set captcha.jpg to show up in the CAPTCHA dialog
 			sc.ui.label_captcha.setPixmap(pixmap)
+
 			def getcaptcha():
 				self.captchatext = unicode(sc.ui.lineEdit_captcha.text())
 			QtCore.QObject.connect(sc.ui.buttonBox_Captcha, QtCore.SIGNAL("accepted()"), getcaptcha)
@@ -280,12 +319,6 @@ class provider_o2online_de(provider):
 			if re_fail.search(str(response)):
 				return error(self, MainWindow, False, 'Step 3: Probably wrong login parameters, check your account setup')
 
-			#------------------------------------
-			#Check for CAPTCHA request
-			#------------------------------------
-			UpdatePbar(self, MainWindow, app, "4", "Step 4: CAPTCHA-Check")
-			self.check_captcha(response)
-
 			#------------------------------------
 			# Open the EMAIL-Center
 			#------------------------------------
@@ -388,7 +421,7 @@ class provider_o2online_de(provider):
 # Vodafone (NL) Provider written by Shane Kerr <shane@time-travellers.org>
 class provider_vodafone_nl(provider):
 	def getName(self):
-		return 'Vodefone (Nederland)'
+		return 'Vodafone (Nederland)'
 	def hasAdvancedFeatures(self):
 		return False
 	def chars_per_sms(self):
@@ -893,3 +926,322 @@ class provider_Telbo(provider):
 		else:
 		    return False, 'Failed. Provider responded: %s' %(result)
 		MainWindow.Stat.close()
+
+# Vodafonesms 1.0
+# http://www.vodafonesms.cz
+# vodafonesmscz Provider written by Bjoern Olausson <maemo@olausson.de>
+class provider_vodafonesmscz(provider):
+	def getName(self):
+		return 'Vodafonesms (Czech)'
+	def hasAdvancedFeatures(self):
+		return True
+	def AdvModName(self):
+		return 'vodafonesmscz'
+	def chars_per_sms(self):
+		return 160, 760
+	def num_regexp(self, num):
+		# Remove everything except numbers and "+"
+		num = re.sub(r'[^(0-9)|\+]', '', num)
+		#Remove "+420" from the number since it is already filled in the webform.
+		num = num.lstrip("+420")
+		return True, num, 'Valid Number'
+
+	def SetupGUI(self, MainWindow):
+		conn = sqlite3.connect('www2sms.db')
+		c = conn.cursor()
+		ALIAS = unicode(MainWindow.AccSet.ui.lineEdit_SetupNewAlias.text())
+		ADVANCED = ""
+		try:
+			ADVANCED = str(c.execute('select advanced FROM login WHERE alias=?', (ALIAS,)).fetchall()[0][0])
+		except sqlite3.OperationalError:
+			pass
+		if ADVANCED != "None":
+			MainWindow.AdvSet.ui.lineEdit_AdvancedText.setText(ADVANCED)
+		c.close()
+		conn.close()
+
+		def on_AdvSetSave_clicked():
+			ALIAS = unicode(MainWindow.AccSet.ui.lineEdit_SetupNewAlias.text())
+			conn = sqlite3.connect('www2sms.db')
+			c = conn.cursor()
+			DB_SENDER = str(c.execute('select advanced FROM login WHERE alias=?', (ALIAS,)).fetchall()[0][0])
+			SENDER = str(MainWindow.AdvSet.ui.lineEdit_AdvancedText.text())
+			if SENDER != "":
+			    if DB_SENDER == "None" or SENDER != DB_SENDER :
+				c.execute('UPDATE login SET advanced=? WHERE alias=?', (SENDER, ALIAS))
+				conn.commit()
+			c.close()
+			conn.close()
+			# Let the user know that we have saved the the account/changes
+			QtGui.QMessageBox.information(MainWindow.AdvSet, 'www2sms', "saved")
+
+		QtCore.QObject.connect(MainWindow.AdvSet.ui.pushButton_Save, QtCore.SIGNAL("clicked()"), on_AdvSetSave_clicked)
+
+	def get_advanced_setting(self, MainWindow):
+		# Read Sender
+		SENDER = MainWindow.AdvSet.ui.lineEdit_AdvancedText.text()
+		return SENDER
+
+	def sendSMS(self, MainWindow, app, NUMBER, SMSTEXT, LOGIN, PASSWORD):
+		SENDER = self.get_advanced_setting(MainWindow)
+		if SENDER == "":
+			return False, 'A sender is required in the advanced Setup'
+		MainWindow.Stat.show()
+		MainWindow.Stat.ui.progressBar.setMinimum(0)
+		MainWindow.Stat.ui.progressBar.setMaximum(5)
+		#API URL
+		BASEURL = "http://www.vodafonesms.cz/"
+		URL = "%sindex.php?locale=en" %(BASEURL)
+		#Remove invalid chars from message
+		MESSAGE = re.sub(r'\^|\{|\}|\[|\]|\~|\\|\|', '', SMSTEXT)
+		#Assemble REQUEST
+		# Instanziate a Browser
+		UpdatePbar(self, MainWindow, app, "1", "Step 1: Initiating the browser")
+		try:
+			br = mechanize.Browser()
+		except Exception, e:
+			return error(self, MainWindow, e, "Step 1: Failed to initiate the Browser")
+
+		br.set_handle_gzip(False)
+		br.set_handle_referer(True)
+		br.set_handle_redirect(True)
+		br.set_handle_equiv(True)
+		br.set_handle_robots(False)
+
+		# Follows refresh 0 but not hangs on refresh > 0
+		br.set_handle_refresh(mechanize._http.HTTPRefreshProcessor(), max_time=1)
+
+		#br.addheaders = [('User-agent', 'Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.6) Gecko/20100105 Gentoo Firefox/3.5.6')]
+		# Opera
+		br.addheaders = [('User-agent', 'Opera/9.80 (X11; Linux x86_64; U; en) Presto/2.5.24 Version/10.54 Gentoo')]
+
+		UpdatePbar(self, MainWindow, app, "2", "Step 2: Open the website")
+		try:
+			br.open(URL)
+			response = br.response().read()
+		except Exception, e:
+			return error(self, MainWindow, e, 'Step 2: Failed to open %s' %(URL))
+
+		UpdatePbar(self, MainWindow, app, "3", "Step 3: Fill SMS-Form")
+		try:
+			br.select_form(name='smsform')
+		except Exception, e:
+			return error(self, MainWindow, e, "Step 3: Faild to fill the SMS-Form")
+		br.form["number"] = NUMBER
+		MyNumber = self.get_advanced_setting(MainWindow)
+		br.form["mynumber"] = MyNumber
+		br.form["sender"] = LOGIN
+		br.form["message"] = MESSAGE
+		#
+		# Parse the response for the CAPTCHA
+		#
+		UpdatePbar(self, MainWindow, app, "4", "Step 4: Solve CAPTCHA")
+		# Search response for the CAPTCHA
+		captchatext = SearchCAPTCHA(self, BASEURL, response, ""r'imgcode.php\?id='"", "img","src")
+		# Fill the form with the user decrypted CAPTCHA
+		try:
+			br.form["pictogram"] = captchatext
+		except AttributeError:
+			return error(self, MainWindow, e, 'Canceled CAPTCHA')
+		#
+		# Done with the CAPTCHA stuff
+		#
+		UpdatePbar(self, MainWindow, app, "5", "Step 5: Send SMS")
+		try:
+			br.submit()
+		except Exception, e:
+			return error(self, MainWindow, e, 'Step 5: Communication Error')
+		response = br.response().read()
+
+		# Error checking
+		if re.search('Your message has been accepted', response):
+			return True, 'Success. Provider responded:\nYour message has been accepted'
+		elif re.search('You entered the wrong number of a sender', response):
+			return False, 'Failed. Provider responded:\nYou entered the wrong number of a sender'
+		if re.search('You entered the wrong number of a recipient', response):
+			return False, 'Failed. Provider responded:\nYou entered the wrong number of a recipient'
+		elif re.search('You failed to copy the number from the picture properly', response):
+			return False, 'Failed. Provider responded:\nYou failed to copy the number from the picture properly'
+		else:
+			f = open("/home/user/MyDocs/vodafonesms.cz.html", 'w')
+			f.write(response)
+			f.close()
+			return False, 'Failed. Unknown Error!\nPlease see /home/user/MyDocs/vodafonesms.cz.html for the reason'
+		MainWindow.Stat.close()
+
+# Arcor (Germany) 1.0
+# http://https://www.arcor.de/ums/ums_neu_sms.jsp
+# Arcor (Germany) Provider written by Nico Lehmann <kingpin-@gmx.de>
+class provider_arcor_de(provider):
+	def getName(self):
+		return 'Arcor (Germany)'
+	def hasAdvancedFeatures(self):
+		return False
+	def chars_per_sms(self):
+		return 142, 142
+
+	def num_regexp(self, num):
+		# Remove everything except numbers and "+"
+		num = re.sub(r'[^(0-9)|\+]', '', num)
+		num = re.sub(r'^0{1}', '+49', num)
+		num = re.sub(r'^0{2}', '+', num)
+		# Check if the number matches either +491732332 or 01732332
+		if re.match('^(\+[1-9][0-9]+)', num):
+		    return True, num, 'Valid Number'
+		else:
+		    return False, num, 'Invalid formated number,'
+
+	def sendSMS(self, MainWindow, app, NUMBER, SMSTEXT, LOGIN, PASSWORD):
+		MainWindow.Stat.show()
+		MainWindow.Stat.ui.progressBar.setMinimum(0)
+		MainWindow.Stat.ui.progressBar.setMaximum(7)
+
+		# set up our browser
+		UpdatePbar(self, MainWindow, app, "1", "Initiating the browser")
+		br = mechanize.Browser()
+
+
+		# login
+		UpdatePbar(self, MainWindow, app, "2", "Connecting to login page")
+		try:
+			br.open('https://www.arcor.de/ums/ums_neu_sms.jsp')
+		except Exception, e:
+			return error(self, MainWindow, e, "Step 2: Failed to open site")
+
+		UpdatePbar(self, MainWindow, app, "3", "Logging in")
+		br.select_form(name='login')
+		br.form['user_name'] = LOGIN
+		br.form['password'] = PASSWORD
+		try:
+			br.submit()
+		except Exception, e:
+			return error(self, MainWindow, e, "Step 3: Failed to submit login credentials")
+
+		# verify our login worked and return error if not
+		UpdatePbar(self, MainWindow, app, "4", "Checking login")
+		try:
+			br.find_link(text_regex="Logout")
+		except Exception, e:
+			return error(self, MainWindow, e, "Step 4: Login failed, probably wrong login/password")
+
+		# Skip the login promo. Since we have no permanent cokies I guess we have to do it every time
+		UpdatePbar(self, MainWindow, app, "5", "Skip first login promo")
+		try:
+			br.select_form(name='loginpromoform')
+			br.submit(name='weiter', label='Weiter')
+		except mechanize._mechanize.FormNotFoundError:
+			pass
+
+		UpdatePbar(self, MainWindow, app, "6", "Sending SMS")
+		try:
+			br.select_form(name='smsform')
+			br.form['empfaengerAn'] = NUMBER
+			br.form['nachricht'] = SMSTEXT.encode(br.encoding())
+			br.submit(name='senden', label='Senden')
+		except Exception, e:
+			print e
+			return error(self, MainWindow, e, "Step 6: Error submitting SMS form")
+
+		# check to see if our send worked
+		UpdatePbar(self, MainWindow, app, "7", "Confirming SMS sent")
+		response = br.response().read()
+		if re.search(r'gesendet!', response):
+			return True, "SMS was sent."
+		else:
+			return False, 'Message not sent'
+		time.sleep(5)
+		MainWindow.Stat.close()
+
+# vodafone.it 0.8 alpha
+# Vodafone (IT) Provider written by
+#class provider_vodafone_it(provider):
+	#def getName(self):
+		#return 'Vodafone (Italy)'
+	#def hasAdvancedFeatures(self):
+		#return False
+	#def chars_per_sms(self):
+		#return 360, 360
+	#def num_regexp(self, num):
+	    	## Remove everything except numbers and "+"
+		#num = re.sub(r'[^(0-9)|\+]', '', num)
+		##Since we can send SMS anywhere it does not make sense to check for vlidity
+		#return True, num, 'Valid Number'
+
+	#def sendSMS(self, MainWindow, app, NUMBER, SMSTEXT, LOGIN, PASSWORD):
+
+                #MainWindow.Stat.show()
+		#MainWindow.Stat.ui.progressBar.setMinimum(0)
+		#MainWindow.Stat.ui.progressBar.setMaximum(9)
+
+		## set up our browser
+		#UpdatePbar(self, MainWindow, app, "1", "Initiating the browser")
+		#br = mechanize.Browser()
+
+		## login
+		#try:
+			#UpdatePbar(self, MainWindow, app, "2", "Connecting to login page")
+			#br.open('http://www.vodafone.it/190/trilogy/jsp/home.do?tabName=Privati&ty_skip_md=true')
+			#UpdatePbar(self, MainWindow, app, "3", "Logging in")
+			#br.select_form(name='loginForm')
+			#br.form['username'] = LOGIN
+			#br.form['password'] = PASSWORD
+			#br.submit()
+		#except:
+			#return False, "Error logging in"
+
+		## send the SMS
+
+   	        #UpdatePbar(self, MainWindow, app, "4", "Opening SMS page")
+		#try:
+			#br.open('http://www.areaprivati.vodafone.it/190/trilogy/jsp/programView.do?ty_target_type=1&ty_program_ctx=/fsms&ty_program_page=/precheck.do&ty_nocache=true&pageTypeId=9604&channelId=-18126&programId=10384&ty_key=fsms_hp&ipage=next')
+		#except:
+			#return False, 'Error opening free SMS web page'
+   	        #UpdatePbar(self, MainWindow, app, "5", "Checking maximum SMS per day")
+		#response = br.response().read()
+		#if re.search(r'Je hebt het maximum aantal gratis SMS\'jes bereikt voor vandaag.', response):
+			#return False, "Provider responded: Je hebt het maximum aantal gratis SMS'jes bereikt voor vandaag."
+
+   	        #UpdatePbar(self, MainWindow, app, "6", "Sending SMS")
+   	        ## Now we fill in all required stuff
+		#br.select_form(name='fsmsMessageForm')
+		#br.form['receiverNumber'] = number
+		#br.form['message'] = SMSTEXT.encode(br.encoding())
+		## Submitt the SMS and store the response since we want to search the CAPTCHA Image
+		## in the response. I hope it wors this way, otherwise we have to use meachnize
+		## to read the image and go back (like I did on o2online)
+		#br.submit()
+		#response = br.response().read()
+		##------------------------------------
+		##Check for CAPTCHA request
+		##------------------------------------
+		#UpdatePbar(self, MainWindow, app, "6", "CAPTCHA-Check")
+		## Here we need the BASEURL which is later completet with the image part XYZ that
+		## comes from the <img src="XYZ">
+		#BASEURL = "http://SOMETHING/"
+		## Here goes the regex which is applied on all <img src=""> elements to find the
+		## one containing the CAPTCHA URL
+		#MyREGEX = ""r'imgcode.php\?id='""
+		## Lets run the search on the response
+		## You uncomment the print statement on line 36 to see all found elements
+		#captchatext = SearchCAPTCHA(self, BASEURL, response, ""r'imgcode.php\?id='"", "img","src")
+		## Now that we have the CAPTCHA use mechanize to fill the form and submit it
+		#br.select_form(name='XXXXX')
+		#br.form['YYYY'] = captchatext
+		#br.submit()
+		## Again, store the respons so we can have a look at it
+		#response = br.response().read()
+		## Save it to a file 
+		#f = open("/home/user/vodafoneit.html", 'w')
+		#f.write(response)
+		##------------------------------------
+		##Done with check for CAPTCHA request
+		##------------------------------------
+		#UpdatePbar(self, MainWindow, app, "7", "Confirming SMS sent")
+		#if re.search(r'Je sms\'je is verstuurd.', response):
+			#return True, "Provider responded: Je sms'je is verstuurd."
+		#elif re.search(r'Je hebt het maximum aantal gratis SMS\'jes voor vandaag verstuurd.', response):
+			#return False, "Provider responded: Je hebt het maximum aantal gratis SMS'jes voor vandaag verstuurd."
+		#else:
+			#return False, 'Message not sent'
+        	#MainWindow.Stat.close()
+
